#ifndef COM_SLAVE_H_	
																											//    
#define COM_SLAVE_H_

#ifndef F_CPU
# warning "F_CPU not defined. Frequency by default 14.7456 MHz"
#define F_CPU 14745600UL																					/*     	*/																						 
#endif																										

#include <avr/eeprom.h>																						//     EEPROM
#include <avr/iom128.h>																						//  /
#include <avr/interrupt.h>																					//   

#include <string.h>																							/*     */

#include "COM.h"																							//    COM 
//------------------------------------------------------------------------------------------
//========================    COM  ==========================
//------------------------------------------------------------------------------------------

#define  ON 1																								//ON -   Slave
#define  OFF 0																								//OFF -  Slave  

#ifndef SLAVE_COM1
# warning "SLAVE_COM1 not defined. Mode by default set OFF"
#define SLAVE_COM1 OFF																						/*      COM1.
																											        .
																											      ON  OFF*/
#endif
#ifndef SLAVE_COM2
# warning "SLAVE_COM2 not defined. Mode by default set OFF"
#define SLAVE_COM2 OFF																						/*      COM2
																											        .
																											      ON  OFF*/
#endif

#if SLAVE_COM1 > 1
#error SLAVE_COM1 The port value can only have an ON or OFF state											// ,       Slave
#endif
#if SLAVE_COM2 > 1
#error SLAVE_COM2 The port value can only have an ON or OFF state											// ,       Slave
#endif
//------------------------------------------------------------------------------------------
//================================    ================================
//------------------------------------------------------------------------------------------
//libs errors
#define COM_ERROR 0xFF																						//     
#define REG_ERROR 0xFE																						//     
#define POSITION_ERROR 0xFC																					//  (lib error)
#define VALUE_ERROR 0xFB																					// 
#define LENGTH_ERROR 0xFA																					//           

//libs ok
#define GOOD 0																								//  
#define ERROR 1																								//  
#define MODBUS_NO_DATA_RECIVED	0xF9																		//  modbus  

//error modbus response
#define MODBUS_FUNC_ERROR 0x01																				//      
#define MODBUS_ADD_ERROR 0x02																				//  (modbus)
#define MODBUS_DATA_ERROR 0x03																				//    

//------------------------------------------------------------------------------------------
//========         COM  ========
//------------------------------------------------------------------------------------------
#if SLAVE_COM1 == ON
	#ifndef SIZE_COIL_COM1
	# warning "SIZE_COIL_COM1 not defined. Size by default set 10"
	#define SIZE_COIL_COM1 10																					/*    Coils  COM1*/
	#endif

	#ifndef SIZE_DISCRETE_INPUT_COM1
	# warning "SIZE_DISCRETE_INPUT_COM1 not defined. Size by default set 10"
	#define SIZE_DISCRETE_INPUT_COM1 10																		   /*    Discrete Input  COM1*/
	#endif
		
	#ifndef SIZE_ANALOG_INPUT_COM1
	# warning "SIZE_ANALOG_INPUT_COM1 not defined. Size by default set 10"
	#define SIZE_ANALOG_INPUT_COM1 10																		   /*    Input Register  COM1*/
	#endif
			
	#ifndef SIZE_ANALOG_OUTPUT_COM1
	# warning "SIZE_ANALOG_OUTPUT_COM1 not defined. Size by default set 10"
	#define SIZE_ANALOG_OUTPUT_COM1 10																		   /*    Holding Register  COM1*/
	#endif
#endif

#if SLAVE_COM2 == ON
	#ifndef SIZE_COIL_COM2
	# warning "SIZE_COIL_COM2 not defined. Size by default set 10"
	#define SIZE_COIL_COM2 10																					/*    Coils  COM2*/
	#endif

	#ifndef SIZE_DISCRETE_INPUT_COM2
	# warning "SIZE_DISCRETE_INPUT_COM2 not defined. Size by default set 10"
	#define SIZE_DISCRETE_INPUT_COM2 10																		   /*    Discrete Input  COM2*/
	#endif

	#ifndef SIZE_ANALOG_INPUT_COM2
	# warning "SIZE_ANALOG_INPUT_COM2 not defined. Size by default set 10"
	#define SIZE_ANALOG_INPUT_COM2 10																		   /*    Input Register  COM2*/
	#endif

	#ifndef SIZE_ANALOG_OUTPUT_COM2
	# warning "SIZE_ANALOG_OUTPUT_COM2 not defined. Size by default set 10"
	#define SIZE_ANALOG_OUTPUT_COM2 10																		   /*    Holding Register  COM2*/
	#endif
#endif

#if SLAVE_COM1 == ON																							// COM1 == ,       
	uint8_t  mas_coil1[SIZE_COIL_COM1] EEMEM;
	uint8_t  mas_discrete_input1[SIZE_DISCRETE_INPUT_COM1];
	uint16_t  mas_analog_input1[SIZE_ANALOG_INPUT_COM1];
	uint16_t  mas_analog_output1[SIZE_ANALOG_OUTPUT_COM1] EEMEM;
#endif
#if SLAVE_COM2 == ON																							// COM2 == ,       
	uint8_t  mas_coil2[SIZE_COIL_COM2] EEMEM;
	uint8_t  mas_discrete_input2[SIZE_DISCRETE_INPUT_COM2];
	uint16_t  mas_analog_input2[SIZE_ANALOG_INPUT_COM2];
	uint16_t  mas_analog_output2[SIZE_ANALOG_OUTPUT_COM2] EEMEM;
#endif

//--------------------------------------------------------------------------------------------
//========================       ========================
//--------------------------------------------------------------------------------------------
#define COIL	0																								//  -  Coils
#define DISCRETE_INPUT	1																						//  -  Discrete Input
#define ANALOG_INPUT	2																						//  -  Input Registers
#define ANALOG_OUTPUT	3																						//  -  Holding Registers

//--------------------------------------------------------------------------------------------
//========================    Slave =============================
//--------------------------------------------------------------------------------------------
uint8_t ADDRESS_COM1 = 1;																							//   Slave (COM1)
uint8_t ADDRESS_COM2 = 2;																							//   Slave (COM2)
//--------------------------------------------------------------------------------------------
//========================     =============================
//--------------------------------------------------------------------------------------------
struct Check
{
	char Adress;																								//     
	char Func;																									//     
	unsigned int SubFunc;																						//       
	unsigned int Amount;																						//       
	char *Data;																									//     
	uint8_t Quantity;																							//      -  
	char byte[256];																								//     
}Package;

//---------------------------------------------------------------------------------------------
//====================================== HELPERS LIB ========================================
//---------------------------------------------------------------------------------------------

//====================================== HANDLER POSITION =====================================
//port -  
//type_register -  
//possition - 
uint8_t Resolve_Position(uint8_t port, int type_register,unsigned int possition){
	if (port > 1) return COM_ERROR;
	switch (port){
		#if SLAVE_COM1 == ON
		case COM_PORT1:
			switch(type_register){
				case COIL:
					if (possition > SIZE_COIL_COM1 - 1) return POSITION_ERROR;
					break;
				case DISCRETE_INPUT:
					if (possition > SIZE_DISCRETE_INPUT_COM1 - 1) return POSITION_ERROR;
					break;
				case ANALOG_INPUT:
					if (possition > SIZE_ANALOG_INPUT_COM1 - 1) return POSITION_ERROR;
					break;
				case ANALOG_OUTPUT:
					if (possition > SIZE_ANALOG_OUTPUT_COM1 - 1) return POSITION_ERROR;
					break;
				default: return REG_ERROR;	
			}
		break;
		#endif
		
		#if SLAVE_COM2 == ON
		case COM_PORT2:
			switch(type_register){
				case COIL:
					if (possition > SIZE_COIL_COM2 - 1) return POSITION_ERROR;
					break;
				case DISCRETE_INPUT:
					if (possition > SIZE_DISCRETE_INPUT_COM2 - 1) return POSITION_ERROR;
					break;
				case ANALOG_INPUT:
					if (possition > SIZE_ANALOG_INPUT_COM2 - 1) return POSITION_ERROR;
					break;
				case ANALOG_OUTPUT:
					if (possition > SIZE_ANALOG_OUTPUT_COM2 - 1) return POSITION_ERROR;
					break;
				default: return REG_ERROR;
			}
			break;
		#endif
		
		default: return COM_ERROR;
	}
	return GOOD;
}

//---------------------------------------------------------------------------------------------
//====================================== HANDLERS WRITE/READ========================================
//---------------------------------------------------------------------------------------------

// ======================================Coil(EEPROM)======================================
//selected_port -  
//possition - 
//value -  (0-1)
uint8_t  Write_COIL(uint8_t selected_port, int possition, uint8_t value){												//     coil single
	if (value != 0 && value != 1) return VALUE_ERROR;
	uint8_t check_position = Resolve_Position(selected_port,COIL, possition);
	if (check_position != GOOD) return check_position;
	switch (selected_port){
		#if SLAVE_COM1 == ON
			case COM_PORT1:
			eeprom_write_byte((void *)&mas_coil1+possition, value);
			break;
		#endif
		#if SLAVE_COM2 == ON
			case COM_PORT2:
				eeprom_write_byte((void *)&mas_coil2+possition, value);
			break;
		#endif
		default: return COM_ERROR;
	}
	return GOOD;
}
//selected_port -  
//data -  
//length -  
uint8_t  Write_multi_COIL(uint8_t selected_port, uint8_t *data, uint16_t length, int possition){										//     coil multi
	switch (selected_port){
		#if SLAVE_COM1 == ON
		case COM_PORT1:
			if (possition > SIZE_COIL_COM1-1) return POSITION_ERROR;
			if (length+possition > SIZE_COIL_COM1 -1) return LENGTH_ERROR;
			for (int i = 0; i < length;i++){
				if (data[i] != 0 && data[i] != 1) return VALUE_ERROR;
			}
			eeprom_write_block((void *)data, (void *)&mas_coil1+possition, length * sizeof(uint8_t));
		break;
		#endif
		#if SLAVE_COM2 == ON
		case COM_PORT2:
			if (possition > SIZE_COIL_COM2-1) return POSITION_ERROR;
			if (length+possition > SIZE_COIL_COM2 -1) return LENGTH_ERROR;
			eeprom_write_block((void *)data, (void *)&mas_coil2+possition, length * sizeof(uint8_t));
		break;
		#endif
		default: return COM_ERROR;
	}
	return GOOD;
}
//selected_port -  
//possition - 
//storage -   
uint8_t Read_COIL(uint8_t selected_port, int possition, uint8_t* storage){												//    coil
	uint8_t check_position = Resolve_Position(selected_port,COIL, possition);
	if (check_position != GOOD) return check_position;
	switch (selected_port){
		#if SLAVE_COM1 == ON
			case COM_PORT1:
				*storage = eeprom_read_byte((void *)&mas_coil1+possition);
			break;
		#endif
		#if SLAVE_COM2 == ON
			case COM_PORT2:
				*storage = eeprom_read_byte((void *)&mas_coil2+possition);
			break;
		#endif
		default: return COM_ERROR;
	}
	return GOOD;
}
// ======================================END Coil======================================

// ================================DISCRETE_INPUT======================================
//selected_port -  
//possition - 
//value -  (0-1)
uint8_t Write_DISCRETE_INPUT(uint8_t selected_port, int possition, uint8_t value){										//    discrete input
	if (value != 0 && value != 1) return VALUE_ERROR;
	uint8_t check_position = Resolve_Position(selected_port,DISCRETE_INPUT, possition);
	if (check_position != GOOD) return check_position;
	switch (selected_port){
		#if SLAVE_COM1 == ON
			case COM_PORT1:
				mas_discrete_input1[possition] = value;
			break;
		#endif
		#if SLAVE_COM2 == ON
			case COM_PORT2:
				mas_discrete_input2[possition] = value;
			break;
		#endif
		default: return COM_ERROR;
	}
	return GOOD;
}

//selected_port -  
//data -  
//length -  
//possition -  
uint8_t  Write_multi_DISCRETE_INPUT(uint8_t selected_port, uint8_t *data, uint16_t length, int possition){						//     coil multi
	switch (selected_port){
		#if SLAVE_COM1 == ON
		case COM_PORT1:
			if (possition > SIZE_DISCRETE_INPUT_COM1-1) return POSITION_ERROR;
			if (length+possition> SIZE_DISCRETE_INPUT_COM1 -1) return LENGTH_ERROR;
			for (int i = 0; i < length;i++){																						//   
				if (data[i] != 0 && data[i] != 1) return VALUE_ERROR;
			}
			for (int i = 0; i < length;i++){																						//  
				mas_discrete_input1[possition+i] = data[i];
			}
		break;
		#endif
		#if SLAVE_COM2 == ON
		case COM_PORT2:
			if (possition > SIZE_DISCRETE_INPUT_COM2-1) return POSITION_ERROR;
			if (length+possition > SIZE_DISCRETE_INPUT_COM2 -1) return LENGTH_ERROR;
			for (int i = 0; i < length;i++){																						//   
				if (data[i] != 0 && data[i] != 1) return VALUE_ERROR;
			}
			for (int i = 0; i < length;i++){																						//  
				mas_discrete_input1[possition+i] = data[i];
			}
		break;
		#endif
		default: return COM_ERROR;
	}
	return GOOD;
}

//selected_port -  
//possition - 
//storage -   
uint8_t Read_DISCRETE_INPUT(uint8_t selected_port, int possition, uint8_t* storage){									//    discrete input
	uint8_t check_position = Resolve_Position(selected_port,DISCRETE_INPUT, possition);
	if (check_position != GOOD) return check_position;
	switch (selected_port){
		#if SLAVE_COM1 == ON
			case COM_PORT1:
				*storage = (uint8_t)mas_discrete_input1[possition];
			break;
		#endif
		#if SLAVE_COM2 == ON
			case COM_PORT2:
				*storage = (uint8_t)mas_discrete_input2[possition];
			break;
		#endif
		default: return COM_ERROR;
	}
	return GOOD;
}
// =============================END DISCRETE_INPUT=================================

// ===============================ANALOG_INPUT=====================================
//selected_port -  
//possition - 
//value - 
uint8_t Write_INPUT_REGISTER(uint8_t selected_port, int possition, uint16_t value){										 //    analog input
	uint8_t check_position = Resolve_Position(selected_port,ANALOG_INPUT, possition);
	if (check_position != GOOD) return check_position;
	switch (selected_port){
		#if SLAVE_COM1 == ON
			case COM_PORT1:
				mas_analog_input1[possition] = value;
			break;
		#endif
		#if SLAVE_COM2 == ON
			case COM_PORT2:
				mas_analog_input2[possition] = value;
			break;
		#endif
		default: return COM_ERROR;
	}
	return GOOD;
}

//selected_port -  
//data -  
//length - 
//possition  
uint8_t Write_multi_INPUT_REGISTER(uint8_t selected_port, uint16_t *data, uint16_t length, int possition){								//    analog output multi
	switch (selected_port){
		#if SLAVE_COM1 == ON
		case COM_PORT1:
			if (possition > SIZE_ANALOG_INPUT_COM1-1) return LENGTH_ERROR;
			if (length+possition > SIZE_ANALOG_INPUT_COM1-1) return LENGTH_ERROR;
			for (int i = 0; i < length;i++){																						//  
				mas_analog_input1[possition+i] = data[i];
			}
		break;
		#endif
		#if SLAVE_COM2 == ON
		case COM_PORT2:
			if (possition > SIZE_ANALOG_INPUT_COM2-1) return LENGTH_ERROR;
			if (length+possition > SIZE_ANALOG_INPUT_COM2-1) return LENGTH_ERROR;
			for (int i = 0; i < length;i++){																						//  
				mas_analog_input2[possition+i] = data[i];
			}
		break;
		#endif
		default: return COM_ERROR;
	}
	return GOOD;
}
//selected_port -  
//possition - 
//storage -   
uint8_t Read_INPUT_REGISTER(uint8_t selected_port, int possition, uint16_t* storage){									//    analog input
	uint8_t check_position = Resolve_Position(selected_port,ANALOG_INPUT, possition);
	if (check_position != GOOD) return check_position;
	switch (selected_port){
		#if SLAVE_COM1 == ON
			case COM_PORT1:
				*storage = (uint16_t)mas_analog_input1[possition];
			break;
		#endif
		#if SLAVE_COM2 == ON
			case COM_PORT2:
				*storage = (uint16_t)mas_analog_input2[possition];
			break;
		#endif
		default: return COM_ERROR;
	}
	return GOOD;
}
// ===============================END ANALOG_INPUT=====================================

// ===============================ANALOG_OUTPUT(EEPROM)================================
//selected_port -  
//possition - 
//value - 
uint8_t Write_HOLDING_REGISTER(uint8_t selected_port, int possition, uint16_t value){										//    analog output single
	uint8_t check_position = Resolve_Position(selected_port,ANALOG_OUTPUT, possition);
	if (check_position != GOOD) return check_position;
	switch (selected_port){
		#if SLAVE_COM1 == ON
			case COM_PORT1:
				eeprom_write_word((void *)&mas_analog_output1+(possition*2), value);
			break;
		#endif
		#if SLAVE_COM2 == ON
			case COM_PORT2:
				eeprom_write_word((void *)&mas_analog_output2+(possition*2), value);
			break;
		#endif
		default: return COM_ERROR;
	}
	return GOOD;
}

//selected_port -  
//data -  
//length -  
//position  
uint8_t Write_multi_HOLDING_REGISTER(uint8_t selected_port, uint16_t *data, uint16_t length, int possition){								//    analog output multi
	switch (selected_port){
		#if SLAVE_COM1 == ON
		case COM_PORT1:
			if (possition > SIZE_ANALOG_OUTPUT_COM1-1) return LENGTH_ERROR;
			if (length+possition > SIZE_ANALOG_OUTPUT_COM1-1) return LENGTH_ERROR;
			eeprom_write_block((void *)data, (void *)&mas_analog_output1+possition*2, length * sizeof(uint16_t));
		break;
		#endif
		#if SLAVE_COM2 == ON
		case COM_PORT2:
			if (possition > SIZE_ANALOG_OUTPUT_COM2-1) return LENGTH_ERROR;
			if (length+possition > SIZE_ANALOG_OUTPUT_COM2-1) return LENGTH_ERROR;
			eeprom_write_block((void *)data, (void *)&mas_analog_output2+possition*2, length * sizeof(uint16_t));		
		break;
		#endif
		default: return COM_ERROR;
	}
	return GOOD;
}

//selected_port -  
//possition - 
//storage -  
uint8_t Read_HOLDING_REGISTER(uint8_t selected_port, int possition, uint16_t* storage){									//    analog output
	uint8_t check_position = Resolve_Position(selected_port,ANALOG_OUTPUT, possition);
	if (check_position != GOOD) return check_position;
	switch (selected_port){
		#if SLAVE_COM1 == ON
			case COM_PORT1:
				*storage = eeprom_read_word((void *)&mas_analog_output1+(possition*2));
			break;
		#endif
		#if SLAVE_COM2 == ON
			case COM_PORT2:
				*storage =	eeprom_read_word((void *)&mas_analog_output2+(possition*2));
			break;
		#endif
		default: return COM_ERROR;
	}
	return GOOD;
}
// ===============================END ANALOG_OUTPUT================================

//---------------------------------------------------------------------------------------------
//======== Write_Slave_mas -        ========
//---------------------------------------------------------------------------------------------
//port -  
//type_register -  
#if (SLAVE_COM1 == 1)||(SLAVE_COM2 ==1)																	//    - COM    Slave
uint8_t Write_Slave_mas(uint8_t port, int type_register)
{
	if (port>1) return COM_ERROR;																		//  
	uint8_t report = 0;																					//    
	
	Package.Data = &Package.byte[0];																	//    Package.byte
	char* curr = (char*)Package.Data;																	// 
		
	switch(type_register)																				//      
	{
		case COIL:
			// 5-      coil'a
			if (Package.Func == 0x05)
			{
				
				if (Package.Amount == 0x0000)															//   
				{
					report = Write_COIL(port, Package.SubFunc, 0);										//   Discrete Output Coils					
					*curr = 0x0000;																		// 00 00    Write_Response_Modbus
				}
				if (Package.Amount == 0xFF00)															//   1  ,  FF00 (hex) = 65280 (dec) -max value unsig uint_16
				{							
					report = Write_COIL(port, Package.SubFunc, 1);										//   Discrete Output Coils
					*curr = 0xFF;																		// FF 00    Write_Response_Modbus
				}
			}
			// 15      coil'  
			if (Package.Func == 0x0F)
			{	
				int j = 0;																				//  
				for (int i = 0; i < Package.Amount; i++)												//  ,    
				{
					uint8_t value = (uint8_t)((*curr >> j) & 0x01);
					report = Write_COIL(port,Package.SubFunc+i,value);
					j++;
					if (j==8)																			//   . ,    ,  
					{
						j = 0;																			//  
						curr++;																			//   
					}
				}
			}
		break;
		case ANALOG_OUTPUT:
		//  0x06  
			if (Package.Func == 0x06)																	//   0x06
			{
				*curr = (Package.Amount>>8);															//        
				curr++;																					//   
				*curr = Package.Amount & 0xFF;															//    
				
				report = Write_HOLDING_REGISTER(port, Package.SubFunc, Package.Amount);					
			}
						//  0x10  
			if (Package.Func == 0x10)
			{

				for (int i = 0; i < Package.Amount; i++)												//  ,     
				{	
					uint16_t value = ((uint16_t)curr[0]<<8) | (curr[1] & 0xFF);	
					report = Write_HOLDING_REGISTER(port, Package.SubFunc+i, value);
					curr+=2;																			//   

				}
			}
			else return MODBUS_FUNC_ERROR;
		break;
		default: return REG_ERROR;																		//   
	}
	if (report != 0) return report;
	return GOOD;
}
//---------------------------------------------------------------------------------------------
//=========== Read_Slave_mas -         ===========
//---------------------------------------------------------------------------------------------
//port -  
//type_register -  
uint8_t Read_Slave_mas(int port, int type_register)
{
	if (port>1) return COM_ERROR;																			//  
	uint8_t report = 0;

	Package.Data = &Package.byte[0];																		//   ,   
	char* current = (char*)Package.Data;																	//   ,   
		
	switch(type_register)																					//      
	{
		//  COIL 0x01 read
		case COIL:
			for (int i=0; i<Package.Amount; i++)															//    
			{	
				uint8_t value_reg = 0;			
				report = Read_COIL(port, Package.SubFunc+i, &value_reg);
				*current |=value_reg<<(i % 8);																//      ,    
					
				if (i%8==7)																					//   . ,    ,  
				{
					current++;																				//   
				}
			}
		break;
		//   0x02 read
		case DISCRETE_INPUT:
			for (int i=0; i<Package.Amount; i++)															//    
			{
				uint8_t value_reg = 0;
				report = Read_DISCRETE_INPUT(port,Package.SubFunc+i, &value_reg);
				*current |=value_reg<<(i % 8);																//      ,    

				if (i%8==7)																					//   . ,    ,  
				{
					current++;																				//   
				}
			}
		break;
		// 0x04   read
		case ANALOG_INPUT:
			for (int i = 0; i < Package.Amount; i++)
			{
				uint16_t value_reg = 0;
 				report = Read_INPUT_REGISTER(port, Package.SubFunc+i, &value_reg);
 					
 				*current |= (value_reg >> 8) & 0xFF;														//  
 				current++;																					//   
 				*current = value_reg & 0xFF;																//  
 				current++;																					//   
			}
		break;
		//  0x03 read
		case ANALOG_OUTPUT:
			for (int i = 0; i < Package.Amount; i++)
			{	
				 uint16_t value_reg = 0;
 				 report = Read_HOLDING_REGISTER(port, Package.SubFunc+i, &value_reg);
					 
 				*current |= (value_reg >> 8) & 0xFF;														//  
 				current++;																					//   
 				*current = value_reg & 0xFF;																//  
 				current++;																					//   
			}
		break;
		default: return REG_ERROR;																			//   
	}
	if (report != 0) return report;
	return GOOD;
}
//------------------------------------------------------------------------------------------------------------------------------
//=========== Check_Slave - ,        ===========
//------------------------------------------------------------------------------------------------------------------------------
//port -  
//RX_Buf -          Write_Request_Modbus
uint8_t Check_Slave(int port)
{
	if (port > 1) return COM_ERROR;																											//   
	
	char RX_Buf[250];																														//    
	char* current = &RX_Buf[1];																												//   
	char* curr = current;																													// 
	
	char warning = 0x00;																													//  ,    
	uint8_t report = 0;

	//  COM_PORT1
	#if SLAVE_COM1 == ON
	if (port == COM_PORT1)
	{
		if(Read_Modbus(COM_PORT1, RX_Buf) ==1)																								//    ,  ,  
		{
			Package.Adress = *curr++;										
			Package.Func = *curr++;											
			Package.SubFunc = *curr++<<8;									
			Package.SubFunc |= *curr++ & 0xFF;								
			Package.Amount = *curr++<<8;									
			Package.Amount |= *curr++ & 0xFF;								
			if (Package.Adress == ADDRESS_COM1)																								//    
			{
				switch(Package.Func)
				{
					//   
					case 0x01:
						if(Package.SubFunc > SIZE_COIL_COM1-1) warning = MODBUS_ADD_ERROR;													//    
						if(Package.SubFunc+Package.Amount > SIZE_COIL_COM1-1 && warning == 0x00) warning = MODBUS_DATA_ERROR;				//     
						if (warning == 0x00){
							report = Read_Slave_mas(COM_PORT1, COIL);																		//   COIL
						}								
					break;
					//   			
					case 0x02:
					Package.Quantity = *curr++;
						if (Package.SubFunc > SIZE_DISCRETE_INPUT_COM1-1) warning = MODBUS_ADD_ERROR;										//    
						if(Package.SubFunc+Package.Amount > SIZE_DISCRETE_INPUT_COM1-1 && warning == 0x00) warning = MODBUS_DATA_ERROR;		//     
						if (warning == 0x00){
							report = Read_Slave_mas(COM_PORT1, DISCRETE_INPUT);																//   DISCRETE_INPUT
						}
					break;
					//   ANALOG_OUTPUT
					case 0x03:
					Package.Quantity = *curr++;
						if (Package.SubFunc > SIZE_ANALOG_OUTPUT_COM1 -1) warning = MODBUS_ADD_ERROR;										//    			
						if ((Package.SubFunc + Package.Amount) > SIZE_ANALOG_OUTPUT_COM1-1 && warning == 0x00) warning = MODBUS_DATA_ERROR;	//     
						if (warning == 0x00){
							report = Read_Slave_mas(COM_PORT1, ANALOG_OUTPUT);																//   ANALOG_OUTPUT
						}
					break;
					//   ANALOG_INPUT
					case 0x04:
					Package.Quantity = *curr++;
						if (Package.SubFunc > SIZE_ANALOG_INPUT_COM1 - 1) warning = MODBUS_ADD_ERROR;										//    
						if ((Package.SubFunc + Package.Amount) > SIZE_ANALOG_INPUT_COM1 - 1 && warning == 0x00) warning = MODBUS_DATA_ERROR;//     		
						if (warning == 0x00){
							report = Read_Slave_mas(COM_PORT1, ANALOG_INPUT);																//   ANALOG_INPUT
						}
					break;
					//   
					case 0x05:
						if (Package.SubFunc > (SIZE_COIL_COM1 - 1)) warning = MODBUS_ADD_ERROR;												//    
						if (warning == 0x00){
							report = Write_Slave_mas(COM_PORT1, COIL);																		//   COIL
						}
					break;
					//  SIZE_ANALOG_OUTPUT
					case 0x06:
						if (Package.SubFunc > (SIZE_ANALOG_OUTPUT_COM1 - 1)) warning = MODBUS_ADD_ERROR;									//    
						if (warning == 0x00){
							report = Write_Slave_mas(COM_PORT1, ANALOG_OUTPUT);																//   ANALOG_OUTPUT
						}
					break;
					//    COIL
					case 0x0F:
						if (Package.SubFunc > SIZE_COIL_COM1 -1) warning = MODBUS_ADD_ERROR;
						if ((Package.SubFunc + Package.Amount) > SIZE_COIL_COM1 -1 && warning == 0x00){										//     
								warning = MODBUS_DATA_ERROR;
						}
						if (warning == 0x00){
								Package.Quantity = *curr++;																					//[7]   -        COIL
								for (int i = 0; i < Package.Quantity; i++)																	// ,     
								{
									Package.byte[i] = *curr++;																				//   
								}
								report = Write_Slave_mas(COM_PORT1, COIL);																	//   COIL
							}
					break;
					//    ANALOG_OUTPUT
					case 0x10:
						if (Package.SubFunc > SIZE_ANALOG_OUTPUT_COM1 -1) warning = MODBUS_ADD_ERROR;
						if ((Package.SubFunc + Package.Amount) > (SIZE_ANALOG_OUTPUT_COM1 - 1) && warning == 0x00){							//     
							warning = MODBUS_DATA_ERROR;
						}
						if (warning == 0x00){
								Package.Quantity = *curr++;																					//[7]   -        ANALOG_OUTPUT
								for (int i = 0; i < Package.Quantity; i++)																	// ,     
								{
									Package.byte[i] = *curr++;																				//   
								}
								report = Write_Slave_mas(COM_PORT1, ANALOG_OUTPUT);															//   ANALOG_OUTPUT
							}
					break;
					default: warning = MODBUS_FUNC_ERROR;																					//   
				}
			}
			if (warning == 0x00){																											//     
					Write_Response_Modbus(COM_PORT1,																						//      
					Package.Adress,
					Package.Func,
					Package.SubFunc,
					Package.Data,
					Package.Amount);
				}else{
					Write_Response_Modbus(COM_PORT1,																						//     
					Package.Adress,
					Package.Func+0x80,
					Package.SubFunc,
					&warning,
					Package.Amount);
				}
			memset(Package.byte,0,256);																										//    
			if (report != 0) return report;
			if (warning != 0x00) return  (uint8_t)warning;
			return GOOD;																													// ,   
		}
	}
	#endif

	#if SLAVE_COM2 == ON
	//  COM_PORT2
	if (port == COM_PORT2)
	{
		if(Read_Modbus(COM_PORT2, RX_Buf) ==1)																								//    ,  ,  
		{
			Package.Adress = *curr++;																										//  
			Package.Func = *curr++;																											// 
			Package.SubFunc = *curr++<<8;																									//  
			Package.SubFunc |= *curr++ & 0xFF;
			Package.Amount = *curr++<<8;																									//- 
			Package.Amount |= *curr++ & 0xFF;
			if (Package.Adress == ADDRESS_COM2)																								//    
			{
				switch(Package.Func)
				{
					//   
					case 0x01:
						if (Package.SubFunc > SIZE_COIL_COM2 -1) warning = MODBUS_ADD_ERROR;												//    
						if ((Package.SubFunc + Package.Amount) > SIZE_COIL_COM2-1 && warning == 0x00)warning = MODBUS_DATA_ERROR;			//     
						if (warning == 0x00){
							report = Read_Slave_mas(COM_PORT2, COIL);																		//   
					}
					break;
					//   
					case 0x02:
					Package.Quantity = *curr++;
						if (Package.SubFunc > SIZE_DISCRETE_INPUT_COM2-1) warning = MODBUS_ADD_ERROR;										//    
						if ((Package.SubFunc + Package.Amount) > SIZE_DISCRETE_INPUT_COM2-1 && warning == 0x00)	warning = MODBUS_DATA_ERROR;//     
						if (warning == 0x00){
							report = Read_Slave_mas(COM_PORT2, DISCRETE_INPUT);																//   
					}						
					break;
					//   
					case 0x03:
					Package.Quantity = *curr++;
						if (Package.SubFunc > SIZE_ANALOG_OUTPUT_COM2-1) warning = MODBUS_ADD_ERROR;										//    
						if ((Package.SubFunc + Package.Amount) > SIZE_ANALOG_OUTPUT_COM2-1 && warning == 0x00)warning = MODBUS_DATA_ERROR;	//     
						if (warning == 0x00){
							report = Read_Slave_mas(COM_PORT2, ANALOG_OUTPUT);																//   
					}
					break;
					//   
					case 0x04:
					Package.Quantity = *curr++;
						if (Package.SubFunc > SIZE_ANALOG_INPUT_COM2-1) warning = MODBUS_ADD_ERROR;											//    
						if ((Package.SubFunc + Package.Amount) > SIZE_ANALOG_INPUT_COM2-1 && warning == 0x00) warning = MODBUS_DATA_ERROR;	//     
						if (warning == 0x00){
							report = Read_Slave_mas(COM_PORT2, ANALOG_INPUT);																//   
					}
					break;
					//   
					case 0x05:
					if (Package.SubFunc > (SIZE_COIL_COM2 - 1))	warning = MODBUS_ADD_ERROR;													//     
						if (warning == 0x00){
							report = Write_Slave_mas(COM_PORT2, COIL);																		//   
					}
					break;
					//   
					case 0x06:
						if (Package.SubFunc > (SIZE_ANALOG_OUTPUT_COM2 - 1)) warning = MODBUS_ADD_ERROR;									//     
						if (warning == 0x00){
							report = Write_Slave_mas(COM_PORT2, ANALOG_OUTPUT);																//   
					}
					break;
					//    
					case 0x0F:
					if (Package.SubFunc > SIZE_COIL_COM2 -1) warning = MODBUS_ADD_ERROR;
					if ((Package.SubFunc + Package.Amount) > SIZE_COIL_COM2-1 && warning == 0x00){																//     
						warning = MODBUS_DATA_ERROR;
						}
					if (warning == 0x00){
						Package.Quantity = *curr++;																							//[7]   -        COIL
						for (int i = 0; i < Package.Quantity; i++)																			// ,     
						{
							Package.byte[i] = *curr++;																						//   
						}
						report = Write_Slave_mas(COM_PORT2, COIL);																			//   
					}
					break;
					//    
					case 0x10:
					if (Package.SubFunc > SIZE_ANALOG_OUTPUT_COM2 -1) warning = MODBUS_ADD_ERROR;
					if ((Package.SubFunc + Package.Amount) > SIZE_ANALOG_OUTPUT_COM2-1  && warning == 0x00){													//     
						warning = MODBUS_DATA_ERROR;
						}
					if (warning == 0x00){
						Package.Quantity = *curr++;																							//[7]   -        ANALOG_OUTPUT
						for (int i = 0; i < Package.Quantity; i++)																			// ,     
						{
							Package.byte[i] = *curr++;																						//   
						}
						report = Write_Slave_mas(COM_PORT2, ANALOG_OUTPUT);																	//    
					}
					break;
					default: warning = MODBUS_FUNC_ERROR;																					//   
				}
			}
		}
		if (warning == 0x00){																												//     
			Write_Response_Modbus(COM_PORT2,																								//      
			Package.Adress,
			Package.Func,
				Package.SubFunc,
				Package.Data,
				Package.Amount);
				}else{
				Write_Response_Modbus(COM_PORT2,																							//     
				Package.Adress,
				Package.Func+0x80,
				Package.SubFunc,
				&warning,
				Package.Amount);
			}
			memset(Package.byte,0,256);																										//    
			if (report != 0) return report;																	
			if (warning != 0x00) return  (uint8_t)warning;
			return GOOD;																													// ,   
		}
	}
	#endif			
	return MODBUS_NO_DATA_RECIVED;																											//  ,  
}

#endif
#endif /* COM_SLAVE_H_ */