/*
        NL.      :
    ;
    ;
    ;
  ;

      COM .        .   Modbus RTU.
*/

#define F_CPU 14745600UL 																		//    

#include "NC8D2R2O.h"																			// 

#include <avr/io.h>																				//  -
#include <avr/eeprom.h>																			//    EEPROM

#define COM_RX_BUFFER_SIZE 32																	//   COM 
#define COM_TX_BUFFER_SIZE 32																	//   COM 

#define COM1_PROTOCOL MODBUS																	//   COM1  Modbus RTU ( - 1  )
#define COM2_PROTOCOL MODBUS																	//   COM2  Modbus RTU ( - 3  )

#include "COM.h"																				//   COM 
#include "LED.h"																				//  
#include "IOPort.h"																				//    /
#include "1Wire.h"																				//    1Wire (  )
#include "RTC.h"																				//     

//----------------------------  EEPROM --------------------------------
unsigned char EE_Address_device EEMEM=0x01;														//   EEPROM
unsigned char EE_COM_Speed EEMEM=0x06;															//   EEPROM

unsigned char EE_Name[] EEMEM="NC8D2R2O";														//   EEPROM
unsigned char EE_Version[] EEMEM="071216";														//   EEPROM

//-----------------------------  RAM ----------------------------------
char Address_device;																			// 
char COM_Speed;																					// 
char Name[10];																					// 
char Version[6];																				// 

char Func;																						// 
int SubFunc;																					// 
int Data;																						// 

char buf[32];																					//     Modbus RTU
int Address_EEP;																				//       EEPROM

int temperature;																				//    
uint8_t hour, min, sec;																			// , , 

//==============    ==============
void Init(void)
{
	int temp;																					// 
	long Speed;																					//   
	
	Address_device=eeprom_read_byte(&EE_Address_device);										//    EEPROM  

	COM_Speed=eeprom_read_byte(&EE_COM_Speed);													//   EEPROM
	
	DDRC=0x00;																					//   
	PORTC=0x00;																					// 
	
	DDRB=0xF0;
	PORTB=0x00;	
	
	Led(LD_ON);
	Led_Red(LD_OFF);
	
	switch (COM_Speed)
	{
		case 0x03: Speed=1200; break;
		case 0x04: Speed=2400; break;
		case 0x05: Speed=4800; break;
		case 0x06: Speed=9600; break;
		case 0x07: Speed=19200; break;
		case 0x08: Speed=38400; break;
		case 0x09: Speed=57600; break;
		case 0x0A: Speed=115200; break;
		default: Speed=9600;
	}
	
	COM_Init(COM_PORT1,Speed);																	//  COM1
	COM_Init(COM_PORT2,Speed);																	//  COM2
	
	RTC_Init();																					// RTC	
	
	Address_EEP=(int)&EE_Name;																	//   EEPROM    

	for ( temp=0; temp<10; temp++)																//    EEPROM
		Name[temp]=eeprom_read_byte(Address_EEP+(uint8_t*)temp);
	
	Address_EEP=(int)&EE_Version;																//   EEPROM    

	for (temp=0; temp<6; temp++)																//  
		Version[temp]=eeprom_read_byte(Address_EEP+(uint8_t*)temp);
}

//===      COM  ===
void Command_decode(char Port)
{
/*
	buf[0]				-   
	buf[1]				-  
	buf[2]				-  
	buf[3]+buf[4]		-   ( )
	buf[5]+buf[6]		-  (  0x05, 0x06  ,  0x01-0x04   )
	buf[7]+buf[8]		- CRC  (   Modbus RTU)
*/
	char temp;																					// 
	int int_temp;																				// 
	
	if (buf[1]==Address_device) 																//  
	{
		Func=buf[2];																			//  
		switch(Func)																			//  
		{
		//    (0x03)
		case 0x03:
			SubFunc=(buf[3]<<8)|buf[4];															//  
			Data=(buf[5]<<8)|buf[6];															// 
			
			switch (SubFunc)																	//  
			{
//--------------------------     ----------------------------
			case 0x0000:
				if (Data!=1)																	//  -    
				{
					buf[0]=0x03;
					Write_Response_Modbus(Port,Address_device,Func | 0x80,SubFunc,buf,1);		// ""
					break;
				}

				buf[0]=0;
				buf[1]=Input_Read_All() & 0x00FF;

//				buf[1]=Input_Read(0);
//				buf[1]|=Input_Read(1)<<1;
//				buf[1]&=~(Input_Read(1)<<1) ^ 0x02;
//				buf[1]|=Input_Read(2)<<2;
//				buf[1]&=~(Input_Read(2)<<2) ^ 0x04;
//				buf[1]|=Input_Read(3)<<3;
//				buf[1]&=~(Input_Read(3)<<3) ^ 0x08;
//				buf[1]|=Input_Read(4)<<4;
//				buf[1]&=~(Input_Read(4)<<4) ^ 0x10;
//				buf[1]|=Input_Read(5)<<5;
//				buf[1]&=~(Input_Read(5)<<5) ^ 0x20;
//				buf[1]|=Input_Read(6)<<6;
//				buf[1]&=~(Input_Read(6)<<6) ^ 0x40;
//				buf[1]|=Input_Read(7)<<7;
//				buf[1]&=~(Input_Read(7)<<7) ^ 0x80;

				Write_Response_Modbus(Port,Address_device,Func,SubFunc,buf,1);
				break;
				
//--------------------------     ---------------------------
			case 0x0010:
				if (Data!=1)																	//  -    
				{
					buf[0]=0x03;
					Write_Response_Modbus(Port,Address_device,Func | 0x80,SubFunc,buf,1);		// ""
					break;
				}

				buf[0]=0;
				buf[1]=Output_Read_All();

//				buf[1]=Output_Read(0);
//				buf[1]|=Output_Read(1)<<1;
//				buf[1]&=~(Output_Read(1)<<1) ^ 0x02;
//				buf[1]|=Output_Read(2)<<2;
//				buf[1]&=~(Output_Read(2)<<2) ^ 0x04;
//				buf[1]|=Output_Read(3)<<3;
//				buf[1]&=~(Output_Read(3)<<3) ^ 0x08;

				Write_Response_Modbus(Port,Address_device,Func,SubFunc,buf,1);
				break;
				
//--------------------------      ----------------------------
			case 0x0020:
				if (Data!=1)																	//  -    
				{
					buf[0]=0x03;
					Write_Response_Modbus(Port,Address_device,Func | 0x80,SubFunc,buf,1);		// ""
					break;
				}

				buf[0]=temperature>>8;
				buf[1]=temperature & 0x00FF;

				Write_Response_Modbus(Port,Address_device,Func,SubFunc,buf,1);
				break;
				
//-------------------------     RTC ---------------------------
			case 0x0030:
				if (Data!=1)																	//  -    
				{
					buf[0]=0x03;
					Write_Response_Modbus(Port,Address_device,Func | 0x80,SubFunc,buf,1);		// ""
					break;
				}

				buf[0]=0;
				buf[1]=hour;

				Write_Response_Modbus(Port,Address_device,Func,SubFunc,buf,1);
				break;
				
//-------------------------     RTC ---------------------------
			case 0x0031:
				if (Data!=1)																	//  -    
				{
					buf[0]=0x03;
					Write_Response_Modbus(Port,Address_device,Func | 0x80,SubFunc,buf,1);		// ""
					break;
				}

				buf[0]=0;
				buf[1]=min;

				Write_Response_Modbus(Port,Address_device,Func,SubFunc,buf,1);
				break;
				
//-------------------------     RTC --------------------------
			case 0x0032:
				if (Data!=1)																	//  -    
				{
					buf[0]=0x03;
					Write_Response_Modbus(Port,Address_device,Func | 0x80,SubFunc,buf,1);		// ""
					break;
				}

				buf[0]=0;
				buf[1]=sec;

				Write_Response_Modbus(Port,Address_device,Func,SubFunc,buf,1);
				break;
				
//------------------------------    -------------------------------
			case 0x00C8:
				if (Data!=4)																	//      
				{
					buf[0]=0x03;
					Write_Response_Modbus(Port,Address_device,Func | 0x80,SubFunc,buf,1);		// ""
					break;
				}
				
				for (int_temp=0; int_temp<8; int_temp++) buf[int_temp]=Name[int_temp];			//  
				Write_Response_Modbus(Port,Address_device,Func,SubFunc,buf,4);
				break;
				
//----------------------------    -----------------------------
			case 0x00D4:
				if (Data!=3)																	//  -    
				{
					buf[0]=0x03;
					Write_Response_Modbus(Port,Address_device,Func | 0x80,SubFunc,buf,1);		// ""
					break;
				}
				
				for (int_temp=0; int_temp<6; int_temp++)										//  
				buf[int_temp]=Version[int_temp];
				Write_Response_Modbus(Port,Address_device,Func,SubFunc,buf,3);
				break;
				
//-------------------------------------   --------------------------------------
			case 0x0200:
				if (Data!=1)																	//  -    
				{
					buf[0]=0x03;
					Write_Response_Modbus(Port,Address_device,Func | 0x80,SubFunc,buf,1);		// ""
					break;
				}

				temp=eeprom_read_byte(&EE_Address_device);										//    EEPROM  
				buf[0]=0;
				buf[1]=temp;
				Write_Response_Modbus(Port,Address_device,Func,SubFunc,buf,1);
				break;
				
//----------------------------------   -----------------------------------
			case 0x0201:
				if (Data!=1)																	//  -    
				{
					buf[0]=0x03;
					Write_Response_Modbus(Port,Address_device,Func | 0x80,SubFunc,buf,1);		// ""
					break;
				}
				
				temp=eeprom_read_byte(&EE_COM_Speed);											//    EEPROM  
				buf[0]=0;
				buf[1]=temp;
				Write_Response_Modbus(Port,Address_device,Func,SubFunc,buf,1);
				break;
				
//----------------------     ----------------------
			default:												
				buf[0]=0x02;																	// ""
				Write_Response_Modbus(Port,Address_device,Func | 0x80,SubFunc,buf,1);
			}
			break;
			
		case 0x06:																				//    (0x06)
			SubFunc=(buf[3]<<8)|buf[4];															//  
			Data=(buf[5]<<8)|buf[6];															// 
			
			switch (SubFunc)																	//      
			{
//------------------------     --------------------------
			case 0x0010:
				Output_Write_All(Data & 0x000F);

//				Output_Write(0,Data & 0x0001);
//				Output_Write(1,(Data & 0x0002)>>1);
//				Output_Write(2,(Data & 0x0004)>>2);
//				Output_Write(3,(Data & 0x0008)>>3);
				
				buf[0]=Data>>8;
				buf[1]=Data & 0x00FF;
				Write_Response_Modbus(Port,Address_device,Func,SubFunc,buf,1);
				break;

//-------------------------------------   --------------------------------------
			case 0x0200:
				if ((Data==0) || (Data>0xF7))													// -     
				{
					buf[0]=0x03;
					Write_Response_Modbus(Port,Address_device,Func | 0x80,SubFunc,buf,1);		// ""
					break;
				}
				
				eeprom_write_byte(&EE_Address_device,Data);										//  EEPROM  
				buf[0]=Data>>8;
				buf[1]=Data & 0x00FF;
				Write_Response_Modbus(Port,Address_device,Func,SubFunc,buf,1);
				break;
				
//------------------------------    -------------------------------
			case 0x0201:
				if ((Data<0x03) || (Data>0x0A))													//       
				{
					buf[0]=0x03;
					Write_Response_Modbus(Port,Address_device,Func | 0x80,SubFunc,buf,1);		// ""
					break;
				}
				
				eeprom_write_byte(&EE_COM_Speed,Data);											//  EEPROM   
				buf[0]=Data>>8;
				buf[1]=Data & 0x00FF;
				Write_Response_Modbus(Port,Address_device,Func,SubFunc,buf,1);
				break;
				
//------------------------------------  RTC -------------------------------------
			case 0x0300:
				RTC_Set_Time(0,0,0);															// ,     
				buf[0]=Data>>8;
				buf[1]=Data & 0x00FF;
				Write_Response_Modbus(Port,Address_device,Func,SubFunc,buf,1);
				break;
				
//----------------------     ----------------------
			default: 																			//    
				buf[0]=0x02;
				Write_Response_Modbus(Port,Address_device,Func | 0x80,SubFunc,buf,1);			// ""
			}
			break;
			
		case 0x10:																				//     (0x10)
			SubFunc=(buf[3]<<8)|buf[4];															//  
			int_temp=(buf[5]<<8)|buf[6];														// - 

			switch (SubFunc)																	//      
			{
//-------------------------------    --------------------------------
			case 0x00C8:
				if (int_temp!=4)																//     
				{
					buf[0]=0x03;
					Write_Response_Modbus(Port,Address_device,Func | 0x80,SubFunc,buf,1);		// ""
					break;
				}
				
				Address_EEP=(int)&EE_Name;														//   EEPROM    

				for (int_temp=0; int_temp<8; int_temp++)										// 
				{
					Name[int_temp]=buf[int_temp+8];
					eeprom_write_byte(Address_EEP+(uint8_t*)int_temp,Name[int_temp]);			//  EEPROM     
				}
				Write_Response_Modbus(Port,Address_device,Func,SubFunc,buf,5);
				break;
				
//----------------------     ----------------------
			default: 																			//    
				buf[0]=0x02;
				Write_Response_Modbus(Port,Address_device,Func | 0x80,SubFunc,buf,1);			// ""
			}
			break;
			
//------------------------     -------------------------
		default:
			buf[0]=0x01;
			Write_Response_Modbus(Port,Address_device,Func | 0x80,SubFunc,buf,1);				// ""
		}
	}
}

//================   ================
int main(void)
{
	uint8_t temp;
	
	Init();
	
	DDRE&=~(1<<PE7);
	PORTE&=~(1<<PE7);
	
	if (!(PINE & (1<<PE7))) 
	{
		Led(LD_OFF);
		Led_Red(LD_ON);
	}
	
	while(1)																					// 
	{
		Temperature_Board_Start();																//   

		for (temp=0; temp<150; temp++)															// 750    	
		{
			if (Read_Modbus(COM_PORT1,buf)==1)	Command_decode(COM_PORT1);						//   COM  ,    
			if (Read_Modbus(COM_PORT2,buf)==1) Command_decode(COM_PORT2);						//   COM  ,   - 
			RTC_Get_Time(&hour,&min,&sec);														//  , , 
			_delay_ms(5);
		}
		
		if (!Temperature_Board_Read(&temperature)) temperature=0x2000;							// 
	}
}
